/* RGB LED Stripxels/Stringxels code

This code is an example of different effects that can be used 
with digitally-controllable RGB LED strips or pixels which are based on the 
WS2801 chip. When using the WS2801 chip you must have a least a 500uS delay 
between data commands for the new command to work. 3 bytes of data are sent in
order to set the intensity of each color channel (Red/Green/Blue). 

We have only programmed for 6 colors, many other colors can be achieved by changing
brightness levels on each channel.

Wiring for Solarbotics strips/pixels:

For Flat-type RGB LED Stringxels (20):
Red Wire = +5V
Yellow Wire = GND
Green Wire = Data
Blue Wire = Clock

For Bullet-type RGB LED Stringxels (40):
Red Wire = +5V
Yellow Wire = GND
Green Wire = Clock
Blue Wire = Data

For RGB LED Stripxels:
Red (attached to black) Wire = +5V
Black Wire = GND
Green Wire = Data (SD)
Red Wire = Clock (CK)

A push button with one side attached to pin 2 and pulled up to 5V through a 
10K resistor and the other side tied to GND will change the mode that is displayed.
If will only work once a sequence has finished so it's a good idea to hold it down.

Effect modes:

Color Wipe 
Rainbow
Color Flash
Larson Scanner
Color Fade (PWM)
Random Color
Larson Scanner + Rainbow


Written by: Alex Filatov for Solarbotics Ltd.
Modified by: Jeremie Boulianne August 2012

*/


byte dataPin = 6;
byte clockPin = 4;
byte numLED = 40; //use 40 for Bullet-type or 20 for Flat-type Stringxels
byte maxBrightness;
byte maxSpeed,color,color1,color2,color3,cx,mode;


void setup() {
    pinMode(dataPin,OUTPUT);
    pinMode(clockPin,OUTPUT);
    pinMode(2,INPUT);
    digitalWrite(2,HIGH);
   maxBrightness=255; //Maximum brightness of channel being effected
   maxSpeed=2;
   randomSeed(analogRead(1));
   cx=0;
   color1=0;
   color2=100;
   color=200;
   mode=0;
}

void loop() { 
 switch (mode) {
    case 0:
    ColorWipe();
    break;
    case 1:
    Rainbow();
    break;
    case 2:
    Larson(); 
    break;
    case 3:
    RandomC(); 
    break;
    case 4:
    ColorFlash(); 
    break;
    case 5:
    Larson2();
    break;
    case 6:
    ColorFade();
    break;
 }
 
 if (digitalRead(2)==LOW) {
  mode=mode+1;
  if (mode==7) {mode=0;}
  delay(500);
 }
 
}

/**********************COLOR WIPE ROUTINE*******************************/
void ColorWipe() {
 color=random(6);
 byte c4=random(50);
 
  for (int k=0; k<numLED; k++) {
   for (int i=0; i<numLED; i++) {
  if (i<k) {
    SetColor();
  }
}
delayMicroseconds(750); //make this delay larger to slow the color wipe down 
                        //you may want to change it to delay()
}
}

/************************RAINBOW ROUTINE********************************/
void Rainbow() {
  for (byte k=0; k<numLED; k++) {
    color=cx; 
    SetColor();
    cx=cx+1;
    if (cx>6) {cx=0;}
   }
   delay(500);
}

/**********************COLOR FLASH ROUTINE********************************/
void ColorFlash() {
    color=random(6);
    for (byte k=0; k<numLED/2; k++) {
     SetColor(); 
   }
    color=random(6);
    for (byte k=numLED/2; k<numLED; k++) {
     SetColor(); 
   }
   delay(2000);
}

/**********************COLOR FADE ROUTINE********************************/
void ColorFade() {
  color=random(6);
  for(byte i=0; i<255; i++) {
   maxBrightness=i;
    for (byte k=0; k<numLED; k++) {
     SetColor(); 
   }
   delay(maxSpeed);
}
delay(500);
   
   for(int i=255; i>-1; i--) {
   maxBrightness=i;
     for (byte k=0; k<numLED; k++) {
     SetColor();   
   }
   delay(maxSpeed);
   }
 delay(500);
maxBrightness=255; 
}

/**********************LARSON SCANNER ROUTINE********************************/
void Larson() {

color=random(6);
 byte c4=random(50);
 
  for (int k=0; k<numLED; k++) {
   for (int i=0; i<numLED; i++) {
  if (i==k || i==k-5 || i==k-10 || i==k-15 || i==k-20) {
    SetColor();
  }
  else
  {
    shiftOut(dataPin,clockPin,MSBFIRST,0);
    shiftOut(dataPin,clockPin,MSBFIRST,0);
    shiftOut(dataPin,clockPin,MSBFIRST,0);
  }
}
delay(c4);
}

for (int k=numLED-1; k>0; k--) {
   for (int i=0; i<numLED; i++) {
  if (i==k || i==k-5 || i==k-10 || i==k-15 || i==k-20) {
   SetColor();
  }
  else
  {
    shiftOut(dataPin,clockPin,MSBFIRST,0);
    shiftOut(dataPin,clockPin,MSBFIRST,0);
    shiftOut(dataPin,clockPin,MSBFIRST,0);
  }
}
delay(c4);
}
}
/**********************RAINBOW LARSON SCANNER ROUTINE***************************/
void Larson2() {

  byte c4=random(50);
  for (int k=0; k<numLED; k++) {
   for (int i=0; i<numLED; i++) {
  if (i==k || i==k-5 || i==k-10 || i==k-15 || i==k-20) {
    color=random(6);
    SetColor();
  }
  else
  {
    shiftOut(dataPin,clockPin,MSBFIRST,0);
    shiftOut(dataPin,clockPin,MSBFIRST,0);
    shiftOut(dataPin,clockPin,MSBFIRST,0);
  }
}
delay(c4);
}

for (int k=numLED-1; k>0; k--) {
   for (int i=0; i<numLED; i++) {
  if (i==k || i==k-5 || i==k-10 || i==k-15 || i==k-20) {
    color=random(6);
    SetColor();
  }
  else
  {
    shiftOut(dataPin,clockPin,MSBFIRST,0);
    shiftOut(dataPin,clockPin,MSBFIRST,0);
    shiftOut(dataPin,clockPin,MSBFIRST,0);
  }
}
delay(c4);
}
}

/**********************RANDOM COLOR ROUTINE********************************/
 void RandomC() {
  for (int i=0; i<numLED; i++) {
  color=random(6);
  SetColor();  
   }
   delay(3000);
 }
 
 /**********************SET COLOR ROUTINE********************************/
 void SetColor() {
   switch (color) {
    case 1:
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      shiftOut(dataPin,clockPin,MSBFIRST,0);
      shiftOut(dataPin,clockPin,MSBFIRST,0);
      break;
    case 2:
      shiftOut(dataPin,clockPin,MSBFIRST,0);
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      shiftOut(dataPin,clockPin,MSBFIRST,0);
      break;
    case 3:
      shiftOut(dataPin,clockPin,MSBFIRST,0);
      shiftOut(dataPin,clockPin,MSBFIRST,0);
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      break;
    case 4:
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      shiftOut(dataPin,clockPin,MSBFIRST,0);
      break;  
    case 5:
      shiftOut(dataPin,clockPin,MSBFIRST,0);
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      break;
    case 6:
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      shiftOut(dataPin,clockPin,MSBFIRST,0);
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      break;
    case 0:
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      shiftOut(dataPin,clockPin,MSBFIRST,maxBrightness);
      break;
  }
 }
